--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       dqs_strobe.vhd
--
--  Description :      This block generates the DQS output signals. The input 
--                is a DQS_RESET signal that holds the DQS DDR flip-flop in
--                reset. This guarantees that the DQS preamble will be met
--                according to the DDR SDRAM specification. The DQS_ENALBE 
--                signal drives the tri-state flip flop. This allows the
--                DQS signal to be active during a write, and tri-stated
--                during a read.
--
--                The DQS output is generated by DDR flip-flops running of
--                CLK0. This generates DQS edge-aligned with the data. A
--                requirement in this configuration is then to insert trace
--                delay on DQS so that it is center aligned with the data (DQ)
--                at the pins of the DDR SDRAM memory device
--
--  Date - revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--****************************************************************************************
library ieee;
use ieee.std_logic_1164.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
entity DQS_strobe is
port (
      DQS_enable : in std_logic;
      DQS_reset  : in std_logic;
      clk         : in std_logic;
      clk180      : in std_logic;
      DDR_DQS    : inout std_logic_vector(3 downto 0);
      DQS        : out std_logic_vector(3 downto 0));
end DQS_strobe;

architecture arc_DQS_Strobe of DQS_strobe is

component v2_dqs_iob
port(
     clk   : in std_logic;
     clk180 : in std_logic;
     DDR_DQS_reset : in std_logic;
     DDR_DQS_enable : in std_logic;
     DDR_DQS        : inout std_logic;
     DQS            : out std_logic);
end component;

signal DQS_enable1 : std_logic := '0';
signal DQS_reset1  : std_logic := '0';
signal DQS_enable2 : std_logic := '0';
signal DQS_enable3 : std_logic := '0';
signal DQS_reset2  : std_logic := '0';
signal DQS_reset3  : std_logic := '0';
signal DQS_enable_int : std_logic := '0';
signal DQS_reset_int  : std_logic := '0';



begin

process(clk)
begin
if clk'event and clk = '1' then
   DQS_enable_int <= DQS_enable;
   DQS_enable1 <= DQS_enable_int;
   DQS_enable2 <= DQS_enable1;
   DQS_enable3 <= not DQS_enable2;
   DQS_reset_int <= DQS_reset;
   DQS_reset1  <= DQS_reset_int;
   DQS_reset2  <= DQS_reset1;
   DQS_reset3  <= DQS_reset2;
end if;
end process;

--******************************************************************************************
--   DQS I/O Block Instantations
--   Instantiate a 'v2_dqs_iob' module for each DQS line required. 
--   The design uses a 32 bit data bus, hence 4 Data Strobe blocks are used
--******************************************************************************************
  
 DQS0 : v2_dqs_iob port map (clk => clk,
                             clk180 => clk180,
                           DDR_DQS_reset => DQS_reset3,
                           DDR_DQS_enable => DQS_enable3,
                           DDR_DQS => DDR_DQS(0),
                           DQS => DQS(0));

 DQS1 : v2_dqs_iob port map  (clk => clk,
                            clk180 => clk180,
                            DDR_DQS_reset => DQS_reset3,
                            DDR_DQS_enable => DQS_enable3,
                            DDR_DQS => DDR_DQS(1),
                            DQS => DQS(1));

 DQS2 : v2_dqs_iob port map (clk => clk,
                           clk180 => clk180,
                           DDR_DQS_reset => DQS_reset3,
                           DDR_DQS_enable => DQS_enable3,
                           DDR_DQS => DDR_DQS(2),
                           DQS => DQS(2));

 DQS3 : v2_dqs_iob port map (clk => clk,
                           clk180 => clk180,
                           DDR_DQS_reset => DQS_reset3,
                           DDR_DQS_enable => DQS_enable3,
                           DDR_DQS => DDR_DQS(3),
                           DQS => DQS(3));
     
                
end arc_DQS_Strobe;