--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :      sync_dqs2clk_DRAM.vhd
--
--  Description :     
--               This module is a synchronizer circuit. It receives data on an 
--               arbitrary clock domain (DQS), which will be immediately captured
--               by the rclk domain. The designer will position the phase shift
--               of the rclk by using the PHASE_SHIFT feature of the DCM. 
--
--               ASYNCHRONOUS DATA TRANSFER
--               The rclk will write data (during a memory read) directly into a 
--               dual-port LUT RAM. Because the control signals for the RAM are 
--               generated on the system clock domain, they must first be transferred
--               to the rclk domain. Because the system clock domain and rclk 
--               domain will have an arbitrary phase relationship, the possibility of
--               setup / hold violations exists. This is protected against by double
--               registering all inputs from the system clock domain on the rclk
--               domain. Similarly, the output data valid signal is also double registered
--               when transferring from the rclk domain to the clk0 domain.
--               The first-stage of the pipeline will be TIG'ed (timing ignore). This
--               FF stage is prefixed by "asynch_<clkname>_signalname", where <clkname>
--               is either rclk or clk0.
--               Data is read out from the LUT RAM on the system clock domain (rclk) on
--               the output_data bus. 
--
--  Date - revision : 05/01/2002
--
--  Author :     Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
--***********************************************************************\

--* 
--***********************************************************************/
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
 
entity sync_dqs2clk is
port(
    input_data_rising  : in std_logic_vector(31 downto 0);
    input_data_falling : in std_logic_vector(31 downto 0);
    input_data_val     : in std_logic;                   -- input data valid
    rclk          : in std_logic;                   -- recapture clock
    data_val_in        : in std_logic;                   -- signal from controller indicating valid data
    output_data        : out std_logic_vector(63 downto 0); 
    clk0         : in std_logic;                   -- system clock that clocks data out from the RAM
    output_data_val    : out std_logic;
    reset                : in std_logic);
end sync_dqs2clk;


architecture arc_sync_dqs2clk of sync_dqs2clk is 

component RAM_8D 
port(
   DPO : out std_logic_vector(7 downto 0);
   A0 : in std_logic;
   A1 : in std_logic;
   A2 : in std_logic;
   A3 : in std_logic;
   D  : in std_logic_vector(7 downto 0);
   DPRA0 : in std_logic;
   DPRA1 : in std_logic;
   DPRA2 : in std_logic;
   DPRA3 : in std_logic;
   WCLK  : in std_logic;
   WE    : in std_logic);
end component;

component RAM_8D_1 
port(
   DPO : out std_logic_vector(7 downto 0);
   A0 : in std_logic;
   A1 : in std_logic;
   A2 : in std_logic;
   A3 : in std_logic;
   D  : in std_logic_vector(7 downto 0);
   DPRA0 : in std_logic;
   DPRA1 : in std_logic;
   DPRA2 : in std_logic;
   DPRA3 : in std_logic;
   WCLK  : in std_logic;
   WE    : in std_logic);
end component;

type deep is range 4 downto 0;
constant depth : integer := 4;

signal output_data_o                : std_logic_vector(31 downto 0);
signal output_data_o_b              : std_logic_vector(31 downto 0);
signal input_data_rising_reg        : std_logic_vector(31 downto 0);
signal input_data_falling_reg       : std_logic_vector(31 downto 0);
signal output_data_mem              : std_logic_vector(31 downto 0);
signal output_data_memb             : std_logic_vector(31 downto 0);
signal output_data_val_out          : std_logic;
signal output_data_reg              : std_logic_vector(63 downto 0);
signal asynch_clk_in_data_val_P1    : std_logic;
signal input_data_val_P2            : std_logic;
signal w_enable                     : std_logic;
signal w_enableb                    : std_logic;
signal w_addr                       : std_logic_vector(1 downto 0);
signal w_addrb                      : std_logic_vector(1 downto 0);
signal asynch_clk_out_addr_int      : std_logic_vector(1 downto 0);
signal asynch_clk_out_addr_intb     : std_logic_vector(1 downto 0);
signal r_addr                       : std_logic_vector(1 downto 0);
signal r_addrb                      : std_logic_vector(1 downto 0);
signal asynch_clk_in_data_val       : std_logic;
signal data_val_P1                  : std_logic;
signal asynch_clk_out_data_val      : std_logic;
signal data_val_out_P1              : std_logic;
signal data_val_out_P2              : std_logic;
signal data_val_out_P3              : std_logic; 
signal data_val_out_P4              : std_logic;  
signal rclkb                   : std_logic; 
signal GND                          : std_logic;  
         
--**********************************************************************\
--   *  Asynchronous clock domain transfer
--   *  Double register all inputs from the CLK domain that will be
--   *  transferred to the rclk domain. 
--   *  Additionally, double register all outputs generated on the rclk
--   *  domain to the clk0 domain.
--   *  This will ensure no mestability issue will occur.
--**********************************************************************/
  
--Transfer input_data_val from the system clock domain to the 
--rclk domain

begin

rclkb <= not rclk;
GND <= '0';

output_data_val <= output_data_val_out;
--output_data <= output_data_reg;

process(rclk)-- enable signal for FIFO
begin
      if rclk'event and rclk = '1' then
  
        asynch_clk_in_data_val_P1 <= input_data_val; 
                input_data_val_P2 <= asynch_clk_in_data_val_P1;

      end if;
end process;

-- Transfer data_val_in from the rclk domain to the clk0 domain
-- for data valid output
process(rclk)
begin
 if rclk'event and rclk = '1' then
     asynch_clk_in_data_val <= data_val_in;
     data_val_P1 <= asynch_clk_in_data_val;
 end if;
end process;


-- Transfer data_val_P1 from the rclk domain to the clk0 domain
process(clk0)
begin
  if clk0'event and clk0 = '1' then
      asynch_clk_out_data_val  <= data_val_P1;
      data_val_out_P1          <= asynch_clk_out_data_val; 
  end if;
end process;

  
--**********************************************************************\
--   *  Pipeline stages
--**********************************************************************/
  
process(rclk)
begin
  if rclk'event and rclk = '1' then
      w_enable <= asynch_clk_in_data_val_P1 or input_data_val_P2 or input_data_val;
  end if;
end process;

process(rclk)
begin
  if rclk'event and rclk = '1' then
      input_data_rising_reg <= input_data_rising;
  end if;
end process;

process(rclk)
begin
  if rclk'event and rclk = '0' then
      input_data_falling_reg <= input_data_falling;

  end if;
end process;

process(clk0,reset)
begin
  if reset = '1' then
        data_val_out_P2          <= '0';
        data_val_out_P3          <= '0';
        data_val_out_P4          <= '0';
        output_data_val_out          <= '0';
  elsif clk0'event and clk0 = '1' then
	data_val_out_P2          <= data_val_out_P1;
	data_val_out_P3          <= data_val_out_P2;
	data_val_out_P4          <= data_val_out_P3;
        output_data_val_out      <= data_val_out_P2;
  end if;
end process;


process(rclk)
begin
  if rclk'event and rclk = '0' then
       w_enableb <= w_enable;
  end if;
end process;

--**********************************************************************\
--  *  DUAL-PORT LUT RAM Address Generation
--**********************************************************************/
 
-- Generate address counter for rising edged input data 

process(rclk, reset)
begin
    if reset = '1' then
      w_addr <= "00";
    elsif rclk'event and rclk = '1' then
       if  w_enable = '1' then
          if w_addr = "11" then
            w_addr <= "00";
          else
            w_addr <= w_addr + "01";
          end if;
       else
          w_addr <= "00";
       end if;
  end if;
end process;



-- Generate address counter for falling edged input data
process(rclk, reset)
begin
   if reset = '1' then
      w_addrb <= "00";
   elsif rclk'event and rclk = '0' then
      if w_enableb = '1' then
        if w_addrb = "11" then
          w_addrb <= "00";
        else
          w_addrb <= w_addrb + "01";
        end if;
      else
        w_addrb <= "00";
      end if;
  end if;
end process;
  
--**********************************************************************\
--  *  Generate logic to pull data out of DUAL-PORT LUT RAM.
--  *  
--  *  Asynchronous clock domain transfer
--  *    Double register the address used to write data into the LUT RAM,
--  *    and then use it to pull data out of the LUT RAM.
--**********************************************************************/
   
-- Transfer the LUT RAM Address for rising edged data from the 
-- rclk domain to the clk0  
process(clk0)
begin
if clk0'event and clk0 = '1' then
    asynch_clk_out_addr_int <= w_addr;
    r_addr   <= asynch_clk_out_addr_int;
end if;
end process;

-- Transfer the LUT RAM Address for falling edged data from the 
-- rclk domain to the clk0  
process(clk0)
begin
   if clk0'event and clk0 = '0' then
        asynch_clk_out_addr_intb  <= w_addrb;
	r_addrb    <= asynch_clk_out_addr_intb;
   end if;
end process;

--**********************************************************************\
--   *  Pipeline output data
--**********************************************************************/
process(clk0)
begin
 if clk0'event and clk0 = '0' then
    output_data_o_b <= output_data_memb;
 end if;
end process;

process(clk0,reset)
begin
 if reset = '1' then
    output_data_o <= "00000000000000000000000000000000";
    output_data  <= "0000000000000000000000000000000000000000000000000000000000000000";
 elsif clk0'event and clk0 = '1' then
    output_data_o <= output_data_mem;
    output_data <= output_data_o & output_data_o_b;
 end if;
end process;


MEM_BYTE3 : RAM_8D port map (  DPO => output_data_mem(31 downto 24),
                               A0  => w_addr(0),
                               A1  => w_addr(1),
                               A2  => GND,
                               A3  => GND,
                               D   => input_data_rising_reg(31 downto 24),
                               DPRA0 => r_addr(0),
                               DPRA1 => r_addr(1),
                               DPRA2 => GND,
                               DPRA3 => GND,
                               WCLK  => rclk,
                               WE    => w_enable);

MEM_BYTE2 : RAM_8D port map (  DPO => output_data_mem(23 downto 16),
                               A0  => w_addr(0),
                               A1  => w_addr(1),
                               A2  => GND,
                               A3  => GND,
                               D   => input_data_rising_reg(23 downto 16),
                               DPRA0 => r_addr(0),
                               DPRA1 => r_addr(1),
                               DPRA2 => GND,
                               DPRA3 => GND,
                               WCLK  => rclk,
                               WE    => w_enable);

MEM_BYTE1 : RAM_8D port map (  DPO => output_data_mem(15 downto 8),
                               A0  => w_addr(0),
                               A1  => w_addr(1),
                               A2  => GND,
                               A3  => GND,
                               D   => input_data_rising_reg(15 downto 8),
                               DPRA0 => r_addr(0),
                               DPRA1 => r_addr(1),
                               DPRA2 => GND,
                               DPRA3 => GND,
                               WCLK  => rclk,
                               WE    => w_enable);
  
MEM_BYTE0 : RAM_8D port map (  DPO => output_data_mem(7 downto 0),
                               A0  => w_addr(0),
                               A1  => w_addr(1),
                               A2  => GND,
                               A3  => GND,
                               D   => input_data_rising_reg(7 downto 0),
                               DPRA0 => r_addr(0),
                               DPRA1 => r_addr(1),
                               DPRA2 => GND,
                               DPRA3 => GND,
                               WCLK  => rclk,
                               WE    => w_enable);

MEM_BYTE3_B : RAM_8D_1 port map (  DPO => output_data_memb(31 downto 24),
                               A0  => w_addrb(0),
                               A1  => w_addrb(1),
                               A2  => GND,
                               A3  => GND,
                               D   => input_data_falling_reg(31 downto 24),
                               DPRA0 => r_addrb(0),
                               DPRA1 => r_addrb(1),
                               DPRA2 => GND,
                               DPRA3 => GND,
                               WCLK  => rclkb,
                               WE    => w_enableb);

MEM_BYTE2_B : RAM_8D_1 port map (  DPO => output_data_memb(23 downto 16),
                               A0  => w_addrb(0),
                               A1  => w_addrb(1),
                               A2  => GND,
                               A3  => GND,
                               D   => input_data_falling_reg(23 downto 16),
                               DPRA0 => r_addrb(0),
                               DPRA1 => r_addrb(1),
                               DPRA2 => GND,
                               DPRA3 => GND,
                               WCLK  => rclkb,
                               WE    => w_enableb);

MEM_BYTE1_B : RAM_8D_1 port map (  DPO => output_data_memb(15 downto 8),
                               A0  => w_addrb(0),
                               A1  => w_addrb(1),
                               A2  => GND,
                               A3  => GND,
                               D   => input_data_falling_reg(15 downto 8),
                               DPRA0 => r_addrb(0),
                               DPRA1 => r_addrb(1),
                               DPRA2 => GND,
                               DPRA3 => GND,
                               WCLK  => rclkb,
                               WE    => w_enableb);

MEM_BYTE0_B : RAM_8D_1 port map (  DPO => output_data_memb(7 downto 0),
                               A0  => w_addrb(0),
                               A1  => w_addrb(1),
                               A2  => GND,
                               A3  => GND,
                               D   => input_data_falling_reg(7 downto 0),
                               DPRA0 => r_addrb(0),
                               DPRA1 => r_addrb(1),
                               DPRA2 => GND,
                               DPRA3 => GND,
                               WCLK  => rclkb,
                               WE    => w_enableb);

end arc_sync_dqs2clk;



  

