--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       v2_ddr_iob.vhd
--
--  Description :     This module creates a single V2 IOB data path. This includes the DDR
--                     output flops with a registered tri-state and the DDR input path.  
--                    
--  Date - revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--
-- pragma translate_off
--library synplify; 
--use synplify.attributes.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;


-- pragma translate_on
--

entity v2_ddr_iob is 
port (
   read_data_rising   : out std_logic;   --Receive data, captured on rising edge
   read_data_falling  : out std_logic;   --Receive data, captured on falling edge
   ddr_dq_inout       : inout std_logic; --Bi-directional SDRAM data bus
   write_data_falling : in std_logic;    --Transmit data, output on falling edge
   write_data_rising  : in std_logic;    --Transmit data, output on rising edge
   wclk               : in std_logic;    --write clock
   clk_rx             : in std_logic;    --Receive clock (DQS)
   write_en_val       : in std_logic;    --Transmit enable
   read_en_val        : in std_logic;    --Receive enable
   reset              : in std_logic); 
    
end v2_ddr_iob;

architecture arc_v2_ddr_iob of v2_ddr_iob is


component FDDRRSE 
port( Q  : out std_logic;
      C0 : in std_logic;
      C1 : in std_logic;
      CE : in std_logic;
      D0 : in std_logic;
      D1 : in std_logic;
      R  : in std_logic;
      S  : in std_logic);
end component;

component FDRE
port(
       D  : in std_logic;
       R  : in std_logic; 
       C  : in std_logic; 
       Q  : out std_logic; 
       CE : in std_logic);
end component;
  
component FDCE 
port(
       D   : in std_logic; 
       CLR : in std_logic; 
       C   : in std_logic; 
       Q   : out std_logic; 
       CE  : in std_logic);
end component;

component OBUFT
port (
       I : in std_logic;
       T : in std_logic;
       O : out std_logic);
end component;

component IBUF
port (
       I : in std_logic;
       O : out std_logic);
end component;
        

--***********************************************************************\
--     Internal signal declaration
--***********************************************************************/
  
signal ddr_en       : std_logic;  -- Tri-state enable signal
signal ddr_dq_q     : std_logic;  -- Data output intermediate signal
signal ddr_dq_o     : std_logic;  -- Data output intermediate signal
signal wclkb      : std_logic;
signal GND          : std_logic;
signal clock_en     : std_logic := '1';
signal enable_b     : std_logic;
signal ddr_dq_ibuf  : std_logic;
signal clk_rx_b     : std_logic;
 
-- synplicity attribute 
--attribute xc_props of DQ_RISE: label is "IOB=TRUE"; 
--attribute xc_props of DQ_FALL: label is "IOB=TRUE"; 

begin


wclkb <= not wclk;
GND    <= '0';
enable_b <= not write_en_val;
clk_rx_b  <= not clk_rx;

 
-- Transmission data path

DDR_OUT : FDDRRSE port map
            (Q  => ddr_dq_q, 
             C0 => wclk, 
             C1 => wclkb, 
             CE => clock_en,
             D0 => write_data_rising, 
             D1 => write_data_falling, 
             R  => GND, 
             S  => GND);
   
DQ_T   :   FDCE port map
           ( D   => enable_b, 
             CLR => reset, 
             C   => wclk, 
             Q   => ddr_en, 
             CE  => clock_en);

  
DQ_OBUFT : OBUFT port map
             ( I => ddr_dq_q,
               T => ddr_en,
               O => ddr_dq_inout);

-- Receive data path

DQ_IBUF :  IBUF port map
             ( I => ddr_dq_inout,
               O => ddr_dq_ibuf);

-- Clock data in
DQ_RISE : FDRE port map
           ( D  => ddr_dq_ibuf, 
             R  => GND, 
             C  => clk_rx,  
             Q  => read_data_rising, 
             CE => read_en_val); 
  
DQ_FALL : FDRE port map
           ( D  => ddr_dq_ibuf, 
             R  => GND, 
             C  => clk_rx_b, 
             Q  => read_data_falling, 
             CE => read_en_val); 


 end arc_v2_ddr_iob;

